--[[
	ItemID v1.5.20200
]]

local ItemID_Version = "1.5";
local WoW_Version = "20300";

local ItemID_Output = {
	ITEMID_CHAN_PARTY,
	ITEMID_CHAN_GUILD,
	ITEMID_CHAN_SAY,
	ITEMID_CHAN_YELL,
	ITEMID_CHAN_RAID,
}

local ItemID_Mode = {
	ITEMID_MODE_FULL,
	ITEMID_MODE_STRING,
	ITEMID_MODE_LINK,
	ITEMID_MODE_IMAGE,
}

function ItemID_load()
	-- Slash Commands
	SLASH_ItemID1 = "/itemid";
	SlashCmdList["ItemID"] = ItemID_commandline;

	ItemID_Msg(ITEMID_VERSION.." "..ItemID_Version.." "..ITEMID_NOWLOADED);
end

-- Message Output
function ItemID_Msg(msg, channel)
	if (msg == nil) then
		return;
	end
	if (channel == ITEMID_CHAN_PARTY) then
		SendChatMessage(msg,"PARTY");
	elseif (channel == ITEMID_CHAN_GUILD) then
		SendChatMessage(msg,"Guild");
	elseif (channel == ITEMID_CHAN_SAY) then
		SendChatMessage(msg,"SAY");
	elseif (channel == ITEMID_CHAN_YELL) then
		SendChatMessage(msg,"YELL");
	elseif (channel == ITEMID_CHAN_RAID) then
		SendChatMessage(msg,"RAID");
	elseif (channel == "Error") then
		DEFAULT_CHAT_FRAME:AddMessage(RED_FONT_COLOR_CODE..ITEMID_TITLE.." "..ITEMID_ERROR..": |r"..msg);
	else
		DEFAULT_CHAT_FRAME:AddMessage(GREEN_FONT_COLOR_CODE..ITEMID_TITLE..": |r"..msg);
	end
end

--== Universal Functions ==--
-- The Error input is for error message overrides. Any input to Error will suppress error message output.

-- mode 0 = base item ID, anything else = Full item ID
function ItemID_GetItemID(itemlink,mode,error)
	local _, Item_Link, _, _, _, _, _, _, _, _ = GetItemInfo(itemlink);
	if (Item_Link == nil) then
		if (error == nil) then
			ItemID_Msg(ITEMID_IDINVALID,"Error");
		end
		return;
	end
	if (mode == "0") then
		local justItemId = string.gsub(Item_Link,".-\124H([^\124]*)\124h.*", "%1");
		Item_Link = strsub(justItemId, string.find(justItemId,":")+1, string.find(justItemId,":",string.find(justItemId,":")+3)-1)
	else
		local _, _, justItemString = string.find(Item_Link, "^|%x+|H(.+)|h%[.+%]");
		Item_Link = justItemString;
	end
	return tostring(gsub(Item_Link, "\124", "\124\124"));
end

function ItemID_ID2Itemlink(itemid,error)
	local Item_Name, Item_Link, _, _, _, _, _, _, _, _ = GetItemInfo(itemid);
	if (Item_Link == Nil) then
		if (error == nil) then
			ItemID_Msg(ITEMID_IDINVALID,"Error");
		end
		return;
	end
	return Item_Link;
end

function ItemID_GetImage(itemlink,error)
	local _, ItemLink, _, _, _, _, _, _, _, itemTexture = GetItemInfo(itemlink);
	if (itemTexture == nil) then
		if (error == nil) then
			ItemID_Msg(ITEMID_LINKINVALID,"Error");
		end
		return;
	end
	return itemTexture;
end

function ItemID_String(itemlink)
	local Item_Name, Item_Link, _, _, _, _, _, _, _, _ = GetItemInfo(itemlink);
	if (Item_Link == nil) then
		if (error == nil) then
			ItemID_Msg(ITEMID_LINKINVALID,"Error");
		end
		return;
	end
	Item_Link2 = gsub(Item_Link, "\124", "\124\124");
	return Item_Link2;
end

-- Itemid-Only Functions
local function ItemID_Cmd_Mode(cmd,mode,channel)
	if (cmd == nil) then
		ItemID_Msg(ITEMID_CMDINVALID,"Error");
		return;
	end
	local result, start;
	if (mode == ITEMID_MODE_FULL) then
		local node = ItemID_GetItemID(cmd,"1",1);
		if node == nil then
			ItemID_Msg(ITEMID_LINKINVALID,"Error");
			return;
		end
		result = ITEMID_ITEMIDFULL..tostring(cmd)..ITEMID_IS..node;
	elseif (mode == ITEMID_MODE_STRING) then
		local node = ItemID_String(cmd,1);
		if node == nil then
			ItemID_Msg(ITEMID_LINKINVALID,"Error");
			return;
		end
		result = ITEMID_ITEMIDSTRING..tostring(cmd)..ITEMID_IS..node;
	elseif (mode == ITEMID_MODE_LINK) then
		local node = ItemID_ID2Itemlink(cmd,1);
		if node == nil then
			ItemID_Msg(ITEMID_IDINVALID,"Error");
			return;
		end
		result = ITEMID_ITEMLINK..node;
	elseif (mode == ITEMID_MODE_IMAGE) then
		local node = ItemID_GetImage(cmd,1);
		if node == nil then
			ItemID_Msg(ITEMID_LINKINVALID,"Error");
			return;
		end
		result = ITEMID_IMAGE..tostring(cmd)..ITEMID_IS..node;
	else
		local node = ItemID_GetItemID(cmd,"0",1);
		if node == nil then
			ItemID_Msg(ITEMID_LINKINVALID,"Error");
			return;
		end
		result = ITEMID_ITEMID..tostring(cmd)..ITEMID_IS..node;
	end
	ItemID_Msg(result, channel);
end

local function ItemID_Cmd_Parse(cmd)
	local mode, output, outcmd, result;
	outcmd = cmd;
	for i=1, getn(ItemID_Mode) do
		local a = ItemID_Mode[i];
		if a == nil then a = ""; end
		if string.find(cmd,a) then
			local input = string.find(cmd,a);
			mode = strsub(cmd,input,input+strlen(a));
			outcmd = strtrim(string.gsub(outcmd,mode,""));
		end
	end
	for k=1, getn(ItemID_Output) do
		local a = ItemID_Output[k];
		if a == nil then a = ""; end
		if string.find(outcmd,a) then
			local input = string.find(outcmd,a);
			output = strsub(outcmd,input,input+strlen(a));
			outcmd = string.gsub(outcmd,output,"");
		end
	end

	if mode == nil then
		mode = "";
	end
	if output == nil then
		output = "";
	end

	-- Used for Debuging purposes.
	-- ItemID_Msg("OutCmd: \""..strtrim(outcmd).."\", Mode: "..strtrim(tostring(mode))..", Output: "..strtrim(tostring(output)));

	ItemID_Cmd_Mode(strtrim(outcmd),strtrim(mode),strtrim(output));
end

-- Command Line handler
function ItemID_commandline(cmd)
	if (cmd == ITEMID_HELP) or (cmd == "") then
		ItemID_Msg(ITEMID_HELP1)
		ItemID_Msg(ITEMID_HELP2);
		ItemID_Msg(ITEMID_HELP3);
		ItemID_Msg(ITEMID_HELP4);
		ItemID_Msg(ITEMID_HELP5);
		ItemID_Msg(ITEMID_HELP6);
	else
		ItemID_Cmd_Parse(cmd);
	end
end